package javax.sql.rowset;

import java.sql.Clob;
import java.sql.Blob;
import java.sql.Timestamp;
import java.sql.Time;
import java.sql.Date;
import java.math.BigDecimal;
import java.lang.reflect.Field;
import java.sql.Types;
import java.sql.SQLException;
import java.io.Serializable;
import javax.sql.RowSetMetaData;

public class RowSetMetaDataImpl implements RowSetMetaData, Serializable
{
    private int colCount;
    private RowSetMetaDataImpl.ColInfo[] colInfo;
    static final long serialVersionUID = 6893806403181801867L;
    
    public RowSetMetaDataImpl() {
    }
    
    private void checkColRange(int i) throws SQLException {
        if (i <= 0 || i > this.colCount) {
            throw new SQLException("Invalid column index :" + i);
        }
        return;
    }
    
    private void checkColType(int n) throws SQLException {
        try {
            Class<Object> obj = Types.class;
            Field[] fields = obj.getFields();
            int i = 0;
            while (i < fields.length) {
                if (fields[i].getInt(obj) == n) {
                    return;
                }
                ++i;
            }
        }
        catch (Exception ex) {
            throw new SQLException(ex.getMessage());
        }
        throw new SQLException("Invalid SQL type for column");
    }
    
    @Override
    public void setColumnCount(int colCount) throws SQLException {
        if (colCount <= 0) {
            throw new SQLException("Invalid column count. Cannot be less or equal to zero");
        }
        this.colCount = colCount;
        if (this.colCount != Integer.MAX_VALUE) {
            this.colInfo = new RowSetMetaDataImpl.ColInfo[this.colCount + 1];
            for (int i = 1; i <= this.colCount; ++i) {
                this.colInfo[i] = new RowSetMetaDataImpl.ColInfo();
            }
        }
        return;
    }
    
    @Override
    public void setAutoIncrement(int n, boolean autoIncrement) throws SQLException {
        this.checkColRange(n);
        this.colInfo[n].autoIncrement = autoIncrement;
    }
    
    @Override
    public void setCaseSensitive(int n, boolean caseSensitive) throws SQLException {
        this.checkColRange(n);
        this.colInfo[n].caseSensitive = caseSensitive;
    }
    
    @Override
    public void setSearchable(int n, boolean searchable) throws SQLException {
        this.checkColRange(n);
        this.colInfo[n].searchable = searchable;
    }
    
    @Override
    public void setCurrency(int n, boolean currency) throws SQLException {
        this.checkColRange(n);
        this.colInfo[n].currency = currency;
    }
    
    @Override
    public void setNullable(int n, int nullable) throws SQLException {
        if (nullable < 0 || nullable > 2) {
            throw new SQLException("Invalid nullable constant set. Must be either columnNoNulls, columnNullable or columnNullableUnknown");
        }
        this.checkColRange(n);
        this.colInfo[n].nullable = nullable;
        return;
    }
    
    @Override
    public void setSigned(int n, boolean signed) throws SQLException {
        this.checkColRange(n);
        this.colInfo[n].signed = signed;
    }
    
    @Override
    public void setColumnDisplaySize(int n, int columnDisplaySize) throws SQLException {
        if (columnDisplaySize < 0) {
            throw new SQLException("Invalid column display size. Cannot be less than zero");
        }
        this.checkColRange(n);
        this.colInfo[n].columnDisplaySize = columnDisplaySize;
        return;
    }
    
    @Override
    public void setColumnLabel(int n, String columnLabel) throws SQLException {
        this.checkColRange(n);
        if (columnLabel != null) {
            this.colInfo[n].columnLabel = columnLabel;
        }
        else {
            this.colInfo[n].columnLabel = "";
        }
    }
    
    @Override
    public void setColumnName(int n, String columnName) throws SQLException {
        this.checkColRange(n);
        if (columnName != null) {
            this.colInfo[n].columnName = columnName;
        }
        else {
            this.colInfo[n].columnName = "";
        }
    }
    
    @Override
    public void setSchemaName(int n, String schemaName) throws SQLException {
        this.checkColRange(n);
        if (schemaName != null) {
            this.colInfo[n].schemaName = schemaName;
        }
        else {
            this.colInfo[n].schemaName = "";
        }
    }
    
    @Override
    public void setPrecision(int n, int colPrecision) throws SQLException {
        if (colPrecision < 0) {
            throw new SQLException("Invalid precision value. Cannot be less than zero");
        }
        this.checkColRange(n);
        this.colInfo[n].colPrecision = colPrecision;
        return;
    }
    
    @Override
    public void setScale(int n, int colScale) throws SQLException {
        if (colScale < 0) {
            throw new SQLException("Invalid scale size. Cannot be less than zero");
        }
        this.checkColRange(n);
        this.colInfo[n].colScale = colScale;
        return;
    }
    
    @Override
    public void setTableName(int n, String tableName) throws SQLException {
        this.checkColRange(n);
        if (tableName != null) {
            this.colInfo[n].tableName = tableName;
        }
        else {
            this.colInfo[n].tableName = "";
        }
    }
    
    @Override
    public void setCatalogName(int n, String catName) throws SQLException {
        this.checkColRange(n);
        if (catName != null) {
            this.colInfo[n].catName = catName;
        }
        else {
            this.colInfo[n].catName = "";
        }
    }
    
    @Override
    public void setColumnType(int n, int colType) throws SQLException {
        this.checkColType(colType);
        this.checkColRange(n);
        this.colInfo[n].colType = colType;
    }
    
    @Override
    public void setColumnTypeName(int n, String colTypeName) throws SQLException {
        this.checkColRange(n);
        if (colTypeName != null) {
            this.colInfo[n].colTypeName = colTypeName;
        }
        else {
            this.colInfo[n].colTypeName = "";
        }
    }
    
    @Override
    public int getColumnCount() throws SQLException {
        return this.colCount;
    }
    
    @Override
    public boolean isAutoIncrement(int n) throws SQLException {
        this.checkColRange(n);
        return this.colInfo[n].autoIncrement;
    }
    
    @Override
    public boolean isCaseSensitive(int n) throws SQLException {
        this.checkColRange(n);
        return this.colInfo[n].caseSensitive;
    }
    
    @Override
    public boolean isSearchable(int n) throws SQLException {
        this.checkColRange(n);
        return this.colInfo[n].searchable;
    }
    
    @Override
    public boolean isCurrency(int n) throws SQLException {
        this.checkColRange(n);
        return this.colInfo[n].currency;
    }
    
    @Override
    public int isNullable(int n) throws SQLException {
        this.checkColRange(n);
        return this.colInfo[n].nullable;
    }
    
    @Override
    public boolean isSigned(int n) throws SQLException {
        this.checkColRange(n);
        return this.colInfo[n].signed;
    }
    
    @Override
    public int getColumnDisplaySize(int n) throws SQLException {
        this.checkColRange(n);
        return this.colInfo[n].columnDisplaySize;
    }
    
    @Override
    public String getColumnLabel(int n) throws SQLException {
        this.checkColRange(n);
        return this.colInfo[n].columnLabel;
    }
    
    @Override
    public String getColumnName(int n) throws SQLException {
        this.checkColRange(n);
        return this.colInfo[n].columnName;
    }
    
    @Override
    public String getSchemaName(int n) throws SQLException {
        this.checkColRange(n);
        String schemaName = "";
        if (this.colInfo[n].schemaName == null) {
            return schemaName;
        }
        schemaName = this.colInfo[n].schemaName;
        return schemaName;
    }
    
    @Override
    public int getPrecision(int n) throws SQLException {
        this.checkColRange(n);
        return this.colInfo[n].colPrecision;
    }
    
    @Override
    public int getScale(int n) throws SQLException {
        this.checkColRange(n);
        return this.colInfo[n].colScale;
    }
    
    @Override
    public String getTableName(int n) throws SQLException {
        this.checkColRange(n);
        return this.colInfo[n].tableName;
    }
    
    @Override
    public String getCatalogName(int n) throws SQLException {
        this.checkColRange(n);
        String catName = "";
        if (this.colInfo[n].catName == null) {
            return catName;
        }
        catName = this.colInfo[n].catName;
        return catName;
    }
    
    @Override
    public int getColumnType(int n) throws SQLException {
        this.checkColRange(n);
        return this.colInfo[n].colType;
    }
    
    @Override
    public String getColumnTypeName(int n) throws SQLException {
        this.checkColRange(n);
        return this.colInfo[n].colTypeName;
    }
    
    @Override
    public boolean isReadOnly(int n) throws SQLException {
        this.checkColRange(n);
        return this.colInfo[n].readOnly;
    }
    
    @Override
    public boolean isWritable(int n) throws SQLException {
        this.checkColRange(n);
        return this.colInfo[n].writable;
    }
    
    @Override
    public boolean isDefinitelyWritable(int n) throws SQLException {
        this.checkColRange(n);
        return true;
    }
    
    @Override
    public String getColumnClassName(int n) throws SQLException {
        String s = String.class.getName();
        switch (this.getColumnType(n)) {
            case 2:
            case 3: {
                s = BigDecimal.class.getName();
                break;
            }
            case -7: {
                s = Boolean.class.getName();
                break;
            }
            case -6: {
                s = Byte.class.getName();
                break;
            }
            case 5: {
                s = Short.class.getName();
                break;
            }
            case 4: {
                s = Integer.class.getName();
                break;
            }
            case -5: {
                s = Long.class.getName();
                break;
            }
            case 7: {
                s = Float.class.getName();
                break;
            }
            case 6:
            case 8: {
                s = Double.class.getName();
                break;
            }
            case -4:
            case -3:
            case -2: {
                s = "byte[]";
                break;
            }
            case 91: {
                s = Date.class.getName();
                break;
            }
            case 92: {
                s = Time.class.getName();
                break;
            }
            case 93: {
                s = Timestamp.class.getName();
                break;
            }
            case 2004: {
                s = Blob.class.getName();
                break;
            }
            case 2005: {
                s = Clob.class.getName();
                break;
            }
        }
        return s;
    }
    
    @Override
    public <T> T unwrap(Class<T> obj) throws SQLException {
        if (this.isWrapperFor(obj)) {
            return obj.cast(this);
        }
        throw new SQLException("unwrap failed for:" + obj);
    }
    
    @Override
    public boolean isWrapperFor(Class<?> clazz) throws SQLException {
        return clazz.isInstance(this);
    }
    
    private class ColInfo implements Serializable
    {
        public boolean autoIncrement;
        public boolean caseSensitive;
        public boolean currency;
        public int nullable;
        public boolean signed;
        public boolean searchable;
        public int columnDisplaySize;
        public String columnLabel;
        public String columnName;
        public String schemaName;
        public int colPrecision;
        public int colScale;
        public String tableName;
        public String catName;
        public int colType;
        public String colTypeName;
        public boolean readOnly;
        public boolean writable;
        static final long serialVersionUID = 5490834817919311283L;
        
        private ColInfo() {
            this.tableName = "";
            this.readOnly = false;
            this.writable = true;
        }
    }
}
